from engines4ai.searxng    import SearXNG
from engines4ai.general    import GeneraleData
from engines4ai.images     import ImageData
from engines4ai.videos     import VideoData
from engines4ai.news       import NewsData
from engines4ai.map        import MapData
from engines4ai.it         import ITData
from engines4ai.science    import ScienceData
from engines4ai.wikipedia  import _wikipedia_engine

from dataclasses import fields, dataclass
from pprint import pprint
import traceback
from typing import Union, List, Literal, Optional


# ------------------------------------------------------------------------------------------
# General engine result parser
# ------------------------------------------------------------------------------------------
def parse_result_to_dataclass(json_data: dict, dc: dataclass):
    field_names = {f.name for f in fields(dc)}
    filtered_data = {k: v for k, v in json_data.items() if k in field_names}
    return dc(**filtered_data)

def search_by(query: str, engines: Union[List[str], str], dc: dataclass, page: int = 1, time_range: Optional[Literal["day", "month", "year"]] = None) -> list:
    engine = SearXNG()
    return_data = []

    try:
        results = engine.search(query=query, engines=engines, pageno=page, time_range=time_range)['results']
        
        for json_data in results:
            parsed = parse_result_to_dataclass(json_data, dc)
            return_data.append(parsed)

    except Exception as e:
        print(f"Error in {engines}_engine: {type(e).__name__} - {str(e)}")
        traceback.print_exc()
        return []

    return return_data

# ------------------------------------------------------------------------------------------
# General
# ------------------------------------------------------------------------------------------
def google_engine(query:str, page:int=1, time_range:Optional[Literal["day", "month", "year"]] = None) -> list[GeneraleData]:
    return search_by(query=query, engines=["google"], dc=GeneraleData, page=page, time_range=time_range)

def bing_engine(query:str, page:int=1, time_range:Optional[Literal["day", "month", "year"]] = None) -> list[GeneraleData]:
    return search_by(query=query, engines=["bing"], dc=GeneraleData, page=page, time_range=time_range)

def wikipedia_engine(query:str, page:int=1) -> list[GeneraleData]:
    return _wikipedia_engine(query=query, page=page)


# ------------------------------------------------------------------------------------------
# Reverse image search
# ------------------------------------------------------------------------------------------
def tineye_engine(url:str) -> list[ImageData]:
    return search_by(query=url, engines=["tineye"], dc=ImageData)

# ------------------------------------------------------------------------------------------
# Images
# ------------------------------------------------------------------------------------------
def google_images_engine(query:str, page:int=1, time_range:Optional[Literal["day", "month", "year"]] = None) -> list[ImageData]:
    return search_by(query=query, engines=["google images"], dc=ImageData, page=page, time_range=time_range)

def adobe_stock_engine(query:str, page:int=1) -> list[ImageData]:
    return search_by(query=query, engines=["adobe stock"], dc=ImageData, page=page)

def unsplash_engine(query:str, page:int=1) -> list[ImageData]:
    return search_by(query=query, engines=["unsplash"], dc=ImageData, page=page)


# ------------------------------------------------------------------------------------------
# Videos
# ------------------------------------------------------------------------------------------
def youtube_engine(query:str, page:int=1, time_range:Optional[Literal["day", "month", "year"]] = None) -> list[VideoData]:
    return search_by(query=query, engines=["youtube"], dc=VideoData, page=page, time_range=time_range)


# ------------------------------------------------------------------------------------------
# News
# ------------------------------------------------------------------------------------------
def bing_news_engine(query:str, page:int=1, time_range:Optional[Literal["day", "month", "year"]] = None) -> list[NewsData]:
    return search_by(query=query, engines=["bing news"], dc=NewsData, page=page, time_range=time_range)

def yahoo_news_engine(query:str, page:int=1, time_range:Optional[Literal["day", "month", "year"]] = None) -> list[NewsData]:
    return search_by(query=query, engines=["yahoo news"], dc=NewsData, page=page, time_range=time_range)


# ------------------------------------------------------------------------------------------
# Maps
# ------------------------------------------------------------------------------------------
def apple_maps_engine(query:str, page:int=1) -> list[MapData]:
    return search_by(query=query, engines=["apple maps"], dc=MapData, page=page)


# ------------------------------------------------------------------------------------------
# IT
# ------------------------------------------------------------------------------------------
def stackoverflow_engine(query:str, page:int=1) -> list[ITData]:
    return search_by(query=query, engines=["stackoverflow"], dc=ITData, page=page)

def github_engine(query:str, page:int=1) -> list[ITData]:
    return search_by(query=query, engines=["github"], dc=ITData, page=page)

# ------------------------------------------------------------------------------------------
# Science
# ------------------------------------------------------------------------------------------
def arxiv_engine(query:str, page:int=1) -> list[ScienceData]:
    return search_by(query=query, engines=["arxiv"], dc=ScienceData, page=page)

def google_scholar_engine(query:str, page:int=1, time_range:Optional[Literal["day", "month", "year"]] = None) -> list[ScienceData]:
    return search_by(query=query, engines=["google scholar"], dc=ScienceData, page=page, time_range=time_range)

# Suspended: access denied
def semantic_scholar_engine(query:str, page:int=1) -> list[ScienceData]:
    return search_by(query=query, engines=["semantic scholar"], dc=ScienceData, page=page)


if __name__ == "__main__":
    results = yahoo_news_engine("deepseek")
    pprint(results[:3])


from engines4ai.searxng import SearXNG

from dataclasses import dataclass, field
from typing import List, Optional
import traceback
from pprint import pprint


@dataclass
class ScienceData:
    authors: List[str] = field(default_factory=list)
    category: str = ""
    comments: Optional[str] = None
    content: str = ""
    doi: Optional[str] = None
    engine: str = ""
    engines: List[str] = field(default_factory=list)
    img_src: str = ""
    journal: Optional[str] = None
    parsed_url: List[str] = field(default_factory=list)
    pdf_url: Optional[str] = None
    positions: List[int] = field(default_factory=list)
    priority: str = ""
    publishedDate: Optional[str] = None
    score: float = 0.0
    tags: List[str] = field(default_factory=list)
    template: str = ""
    thumbnail: str = ""
    title: str = ""
    url: str = ""


def science_engine(query: str, pageno: int = 1, engines: list[str]=[]) -> List[ScienceData]:
    engine = SearXNG()

    try:
        # Only set categories if no specific engines are requested
        search_params = {
            "query": query,
            "pageno": pageno
        }
        
        if engines:
            search_params["engines"] = engines
        else:
            search_params["categories"] = ["science"]
            
        results = engine.search(**search_params)['results']
        science_list = []
        for result in results:
            science = ScienceData(
                authors=result.get("authors", []),
                category=result.get("category", ""),
                comments=result.get("comments", None),
                content=result.get("content", ""),
                doi=result.get("doi", None),
                engine=result.get("engine", ""),
                engines=result.get("engines", []),
                img_src=result.get("img_src", ""),
                journal=result.get("journal", None),
                parsed_url=result.get("parsed_url", []),
                pdf_url=result.get("pdf_url", None),
                positions=result.get("positions", []),
                priority=result.get("priority", ""),
                publishedDate=result.get("publishedDate", None),
                score=result.get("score", 0.0),
                tags=result.get("tags", []),
                template=result.get("template", ""),
                thumbnail=result.get("thumbnail", ""),
                title=result.get("title", ""),
                url=result.get("url", "")
            )
            science_list.append(science)
        return science_list

    except Exception as e:
        print(f"Error in science_engine: {type(e).__name__} - {str(e)}")
        traceback.print_exc()
        return []


if __name__ == "__main__":
    results = science_engine("DeepSeek")
    for paper in results[:3]:
        pprint(paper)



from engines4ai.searxng import SearXNG

from dataclasses import dataclass, field
from typing import List


@dataclass
class ImageData:
    category: str = ''
    content: str = ''
    engine: str = ''
    engines: List[str] = field(default_factory=list)
    img_src: str = ''
    parsed_url: List[str] = field(default_factory=list)
    positions: List[int] = field(default_factory=list)
    priority: str = ''
    resolution: str = ''
    score: float = 0.0
    source: str = ''
    template: str = ''
    thumbnail: str = ''
    thumbnail_src: str = ''
    title: str = ''
    url: str = ''


def image_engine(query: str, 
                 pageno: int = 1,
                 engines: list[str]=[]) -> List[ImageData]:
    engine = SearXNG()

    try:
        # Only set categories if no specific engines are requested
        search_params = {
            "query": query,
            "pageno": pageno
        }
        
        if engines:
            search_params["engines"] = engines
        else:
            search_params["categories"] = ["images"]
            
        results = engine.search(**search_params)['results']
        images = []
        
        for result in results:
            data = ImageData(
                category=result.get("category", ""),
                content=result.get("content", ""),
                engine=result.get("engine", ""),
                engines=result.get("engines", []),
                img_src=result.get("img_src", ""),
                parsed_url=result.get("parsed_url", []),
                positions=result.get("positions", []),
                priority=result.get("priority", ""),
                resolution=result.get("resolution", ""),
                score=result.get("score", 0.0),
                source=result.get("source", ""),
                template=result.get("template", ""),
                thumbnail=result.get("thumbnail", ""),
                thumbnail_src=result.get("thumbnail_src", ""),
                title=result.get("title", ""),
                url=result.get("url", "")
            )
            images.append(data)

        return images

    except Exception as e:
        print(f"Error in image_engine: {type(e).__name__} - {str(e)}")
        raise


if __name__ == "__main__":
    from pprint import pprint
    results = image_engine("banff national park")
    pprint(results[:2])



from engines4ai.searxng import SearXNG

from dataclasses import dataclass, field
from typing import List, Optional
from pprint import pprint
from typing import List
import traceback


@dataclass
class ITData:
    category: str = ''
    content: str = ''
    engine: str = ''
    engines: List[str] = field(default_factory=list)
    homepage: Optional[str] = None
    img_src: str = ''
    license_name: Optional[str] = None
    license_url: Optional[str] = None
    maintainer: str = ''
    package_name: str = ''
    parsed_url: List[str] = field(default_factory=list)
    popularity: int = 0
    positions: List[int] = field(default_factory=list)
    priority: str = ''
    publishedDate: Optional[str] = None
    score: float = 0.0
    source_code_url: str = ''
    tags: List[str] = field(default_factory=list)
    template: str = ''
    thumbnail: str = ''
    title: str = ''
    url: str = ''
    

def it_engine(query: str, pageno: int = 1, engines: list[str]=[]) -> List[ITData]:
    engine = SearXNG()

    try:
        # Only set categories if no specific engines are requested
        search_params = {
            "query": query,
            "pageno": pageno
        }
        
        if engines:
            search_params["engines"] = engines
        else:
            search_params["categories"] = ["it"]
            
        results = engine.search(**search_params)['results']
        it_list = []
        for result in results:
            it = ITData(
                category=result.get("category", ""),
                content=result.get("content", ""),
                engine=result.get("engine", ""),
                engines=result.get("engines", []),
                homepage=result.get("homepage", None),
                img_src=result.get("img_src", ""),
                license_name=result.get("license_name", None),
                license_url=result.get("license_url", None),
                maintainer=result.get("maintainer", ""),
                package_name=result.get("package_name", ""),
                parsed_url=result.get("parsed_url", []),
                popularity=result.get("popularity", 0),
                positions=result.get("positions", []),
                priority=result.get("priority", ""),
                publishedDate=result.get("publishedDate", None),
                score=result.get("score", 0.0),
                source_code_url=result.get("source_code_url", ""),
                tags=result.get("tags", []),
                template=result.get("template", ""),
                thumbnail=result.get("thumbnail", ""),
                title=result.get("title", ""),
                url=result.get("url", "")
            )
            it_list.append(it)
        return it_list

    except Exception as e:
        print(f"Error in it_engine: {type(e).__name__} - {str(e)}")
        traceback.print_exc()
        return []


if __name__ == "__main__":
    it_results = it_engine("openai python sdk")
    for item in it_results[:3]:
        pprint(item)



from engines4ai.searxng import SearXNG

from dataclasses import dataclass
from typing import List
from pprint import pprint
import traceback

@dataclass
class NewsData:
    category: str
    content: str
    engine: str
    engines: List[str]
    img_src: str
    parsed_url: List[str]
    positions: List[int]
    priority: str
    score: float
    template: str
    thumbnail: str
    title: str
    url: str
    

def news_engine(query: str, pageno: int = 1, engines: list[str]=[]) -> List[NewsData]:
    engine = SearXNG()
    
    try:
        # Only set categories if no specific engines are requested
        search_params = {
            "query": query,
            "pageno": pageno
        }
        
        if engines:
            search_params["engines"] = engines
        else:
            search_params["categories"] = ["news"]
            
        results = engine.search(**search_params)['results']
        news_list = []
        for result in results:
            news = NewsData(
                category=result.get("category", ""),
                content=result.get("content", ""),
                engine=result.get("engine", ""),
                engines=result.get("engines", []),
                img_src=result.get("img_src", ""),
                parsed_url=result.get("parsed_url", []),
                positions=result.get("positions", []),
                priority=result.get("priority", ""),
                score=result.get("score", 0.0),
                template=result.get("template", ""),
                thumbnail=result.get("thumbnail", ""),
                title=result.get("title", ""),
                url=result.get("url", "")
            )
            news_list.append(news)
        return news_list

    except Exception as e:
        print(f"Error in news_engine: {type(e).__name__} - {str(e)}")
        traceback.print_exc()
        return []


if __name__ == "__main__":
    news_results = news_engine("DeepSeek AI China")
    for news in news_results[:3]:
        pprint(news)



from engines4ai.searxng import SearXNG

from pprint import pprint
from dataclasses import dataclass, field
from typing import List, Optional, Literal


@dataclass
class GeneraleData:
    category: str = ''
    content: str = ''
    engine: str = ''
    engines: List[str] = field(default_factory=list)
    img_src: str = ''
    parsed_url: List[str] = field(default_factory=list)
    positions: List[int] = field(default_factory=list)
    priority: str = ''
    publishedDate: Optional[str] = None
    score: float = 0.0
    template: str = ''
    thumbnail: str = ''
    title: str = ''
    url: str = ''
    

def general_engine(query: str, pageno: int = 1, engines: list[str]=[], time_range: Literal["day", "month", "year"] = None) -> List[GeneraleData]:
    engine = SearXNG()

    try:
        results = engine.search(query=query, pageno=pageno, engines=engines, time_range=time_range)['results']
        general = []
        for result in results:
            data = GeneraleData(
                category=result.get("category", ""),
                content=result.get("content", ""),
                engine=result.get("engine", ""),
                engines=result.get("engines", []),
                img_src=result.get("img_src", ""),
                parsed_url=result.get("parsed_url", []),
                positions=result.get("positions", []),
                priority=result.get("priority", ""),
                publishedDate=result.get("publishedDate", None),
                score=result.get("score", 0.0),
                template=result.get("template", ""),
                thumbnail=result.get("thumbnail", ""),
                title=result.get("title", ""),
                url=result.get("url", "")
            )
            general.append(data)
        return general

    except Exception as e:
        print(f"Error in general_engine: {type(e).__name__} - {str(e)}")
        raise


if __name__ == "__main__":  
    results = general_engine("openai-python")
    pprint(results[:3])




from engines4ai.searxng import SearXNG

from typing import List
from dataclasses import dataclass, field
import traceback
from pprint import pprint


@dataclass
class VideoData:
    author: str = ''
    category: str = ''
    content: str = ''
    engine: str = ''
    engines: List[str] = field(default_factory=list)
    iframe_src: str = ''
    img_src: str = ''
    length: str = ''
    parsed_url: List[str] = field(default_factory=list)
    positions: List[int] = field(default_factory=list)
    priority: str = ''
    score: float = 0.0
    template: str = ''
    thumbnail: str = ''
    title: str = ''
    url: str = ''


def video_engine(query: str, pageno: int = 1, engines: list[str]=[]) -> List[VideoData]:
    engine = SearXNG()

    try:
        # Only set categories if no specific engines are requested
        search_params = {
            "query": query,
            "pageno": pageno
        }
        
        if engines:
            search_params["engines"] = engines
        else:
            search_params["categories"] = ["videos"]
            
        results = engine.search(**search_params)['results']
        videos = []
        for result in results:
            data = VideoData(
                author=result.get("author", ""),
                category=result.get("category", ""),
                content=result.get("content", ""),
                engine=result.get("engine", ""),
                engines=result.get("engines", []),
                iframe_src=result.get("iframe_src", ""),
                img_src=result.get("img_src", ""),
                length=result.get("length", ""),
                parsed_url=result.get("parsed_url", []),
                positions=result.get("positions", []),
                priority=result.get("priority", ""),
                score=result.get("score", 0.0),
                template=result.get("template", ""),
                thumbnail=result.get("thumbnail", ""),
                title=result.get("title", ""),
                url=result.get("url", "")
            )
            videos.append(data)
        return videos

    except Exception as e:
        print(f"Error in video_engine: {type(e).__name__} - {str(e)}")
        traceback.print_exc()
        return []

if __name__ == "__main__":
    results = video_engine("ChatGPT")
    for video in results[:3]:
        pprint(video)



from engines4ai.searxng import SearXNG
from engines4ai.general import GeneraleData

from typing import List


def _wikipedia_engine(query: str, page: int = 1) -> List[GeneraleData]:
    engine = SearXNG()

    try:
        results = engine.search(query=query, engines=["wikipedia"], pageno=page)['infoboxes']
        wikipedia_results = []
        for result in results:
            data = GeneraleData(
                category=result.get("category", ""),
                content=result.get("content", ""),
                engine=result.get("engine", ""),
                engines=result.get("engines", []),
                img_src=result.get("img_src", ""),
                parsed_url=result.get("parsed_url", []),
                positions=result.get("positions", ""),
                priority=result.get("priority", ""),
                score=result.get("score", 0.0),
                template=result.get("template", ""),
                thumbnail=result.get("thumbnail", ""),
                title=result.get("title", ""),
                url=result.get("urls", "")[0].get("url", ""),
            )
            
            wikipedia_results.append(data)
        return wikipedia_results

    except Exception as e:
        print(f"Error in wikipedia_engine: {type(e).__name__} - {str(e)}")
        raise



from engines4ai.searxng import SearXNG

from dataclasses import dataclass, field
from typing import List, Optional
from pprint import pprint


@dataclass
class MapData:
    title: str = ''
    latitude: float = 0.0
    longitude: float = 0.0
    url: str = ''
    engine: str = ''
    engines: List[str] = field(default_factory=list)
    score: float = 0.0
    geojson: dict = field(default_factory=dict)
    boundingbox: List[float] = field(default_factory=list)
    osm_id: Optional[int] = None
    osm_type: Optional[str] = None
    parsed_url: List[str] = field(default_factory=list)
    positions: List[int] = field(default_factory=list)
    category: str = ''
    template: str = ''
    

def map_engine(query: str, pageno: int = 1, engines: list[str]=[]) -> List[MapData]:
    engine = SearXNG()

    try:
        # Only set categories if no specific engines are requested
        search_params = {
            "query": query,
            "pageno": pageno
        }
        
        if engines:
            search_params["engines"] = engines
        else:
            search_params["categories"] = ["map"]
            
        results = engine.search(**search_params)["results"]
        maps = []

        for result in results:
            data = MapData(
                title=result.get("title", ""),
                latitude=result.get("latitude", 0.0),
                longitude=result.get("longitude", 0.0),
                url=result.get("url", ""),
                engine=result.get("engine", ""),
                engines=result.get("engines", []),
                score=result.get("score", 0.0),
                geojson=result.get("geojson", {}),
                boundingbox=result.get("boundingbox", []),
                osm_id=result.get("osm", {}).get("id"),
                osm_type=result.get("osm", {}).get("type"),
                parsed_url=result.get("parsed_url", []),
                positions=result.get("positions", []),
                category=result.get("category", ""),
                template=result.get("template", "")
            )
            maps.append(data)

        return maps

    except Exception as e:
        print(f"Error in map_engine: {type(e).__name__} - {str(e)}")
        raise


if __name__ == "__main__":
    results = map_engine("Restaurant in tokyo")
    pprint(results)




# https://docs.searxng.org/dev/search_api.html


from pprint import pprint
import requests
from typing import Optional, List, Literal
from dotenv import load_dotenv
import os

class SearXNG:
    def __init__(self, base_url:str=""):
        load_dotenv(override=True)
        self.base_url = os.getenv("SEARXNG_BASE_URL")
        self.api_key = os.getenv("SEARXNG_API_KEY")

        if base_url:
            self.base_url = base_url
        
        self.base_url = self.base_url.rstrip("/") + "/search"

    def search(
        self,
        query: str,
        categories: Optional[
            List[
                Literal[
                    "general", "images", "news", "it", "science",
                    "videos", "music", "social media", "files", "map"
                ]
            ]
        ] = None,
        engines: Optional[List[str]] = None,
        language: Optional[str] = None,
        pageno: int = 1,
        time_range: Optional[Literal["day", "month", "year"]] = None,
        format: Optional[Literal["json", "csv", "rss"]] = "json",
        results_on_new_tab: int = 0,
        image_proxy: Optional[bool] = None,
        autocomplete: Optional[
            Literal[
                "google", "dbpedia", "duckduckgo", "mwmb1", "startpage",
                "wikipedia", "stract", "swisscows", "qwant"
            ]
        ] = None,
        safesearch: Optional[Literal[0, 1, 2]] = None,
        theme: Optional[str] = "simple",
        enabled_plugins: Optional[List[str]] = None,
        disabled_plugins: Optional[List[str]] = None,
        enabled_engines: Optional[List[str]] = None,
        disabled_engines: Optional[List[str]] = None,
    ) -> dict:
        params = {
            "q": query,
            "pageno": pageno,
            "format": format,
            "results_on_new_tab": results_on_new_tab,
            "theme": theme,
        }

        if categories:
            params["categories"] = ",".join(categories)
        if engines:
            params["engines"] = ",".join(engines)
        if language:
            params["language"] = language
        if time_range:
            params["time_range"] = time_range
        if image_proxy is not None:
            params["image_proxy"] = str(image_proxy).lower()
        if autocomplete:
            params["autocomplete"] = autocomplete
        if safesearch is not None:
            params["safesearch"] = safesearch
        if enabled_plugins:
            params["enabled_plugins"] = ",".join(enabled_plugins)
        if disabled_plugins:
            params["disabled_plugins"] = ",".join(disabled_plugins)
        if enabled_engines:
            params["enabled_engines"] = ",".join(enabled_engines)
        if disabled_engines:
            params["disabled_engines"] = ",".join(disabled_engines)
            
        params["key"] = self.api_key

        response = requests.get(self.base_url, params=params)
        response.raise_for_status()
        
        return response.json()


if __name__ == "__main__":
    searxng = SearXNG()
    results = searxng.search(query="google")
    pprint(results)